#include "AISManagedInput.h"

#include "AISUtil.h"

void AISManagedInputPin::poll() {
  uint8_t curState = digitalRead(_pin);
  if (curState != _lastState) {
    _lastTime = millis();
  }
  if ((millis() - _lastTime) > AISDebounceDelay) {
    if (curState != _debouncedState) {
      _debouncedState = curState;
      _sendCommand(curState);
    }
  }
  _lastState = curState;
}

AISTriggerInput::AISTriggerInput(uint8_t pin, OISCommand activeCommand,
				 uint8_t mode,
				 ObjectsInSpaceInterface *serialCallback) {
  _pin = pin;
  _activeCommand = activeCommand;
  // TODO: Why does this throw an error?
  // _inactiveCommand = nullptr;
  _mode = mode;
  _lastTime = 0;
  _serialCallback = serialCallback;
}

AISTriggerInput::AISTriggerInput(uint8_t pin, OISCommand activeCommand,
				 OISCommand inactiveCommand, uint8_t mode,
				 ObjectsInSpaceInterface *serialCallback) {
  _pin = pin;
  _activeCommand = activeCommand;
  _inactiveCommand = inactiveCommand;
  // _mode should contain the value considered inactive.
  if (mode == INPUT_PULLUP) {
    _mode = HIGH;
  } else {
    _mode = LOW;
  }
  _lastTime = 0;
  _serialCallback = serialCallback;
}

void AISTriggerInput::_sendCommand(uint8_t newState) {
  if (_mode ^ newState == 0) {
    if (&_inactiveCommand) {
      _serialCallback->sendCommand(_inactiveCommand);
    }
  } else {
    if (&_activeCommand) {
      _serialCallback->sendCommand(_activeCommand);
    }
  }
}

AISLatchingInput::AISLatchingInput(uint8_t pin, OISCommand firstCommand,
				   OISCommand secondCommand, uint8_t mode,
				   ObjectsInSpaceInterface *serialCallback) {
  _pin = pin;
  _firstCommand = firstCommand;
  _secondCommand = secondCommand;
  // _mode should contain the value considered inactive.
  if (mode == INPUT_PULLUP) {
    _mode = HIGH;
  } else {
    _mode = LOW;
  }
  _lastTime = 0;
  _serialCallback = serialCallback;
}

void AISLatchingInput::_sendCommand(uint8_t newState) {
  if (_mode ^ newState == 0) {
    if (_lastCommandActive) {
      if (&_secondCommand) {
	_serialCallback->sendCommand(_secondCommand);
      }
      _lastCommandActive = false;
    } else {
      if (&_firstCommand) {
	_serialCallback->sendCommand(_firstCommand);
      }
      _lastCommandActive = true;
    }
  }
}
